#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# GUI for autots
#
# Copyright (C) 2008-2009 Maximilian Högner
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

import pygtk
pygtk.require('2.0')

import gtk

import autots,os

# Localisation
_ = autots._

room_windows = {}
drag_pos = {}

def ask(q, w=None):
	d = gtk.MessageDialog(w, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
		gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, q)
	r=d.run()
	d.destroy()

	return r

def complain(c, w=None):
	d = gtk.MessageDialog(w, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, c)
	r=d.run()
	d.destroy()

	return r

class computer_window:
	def __init__(self,computername=None,roomname=None):
		self.computername=computername
		self.roomname=roomname

		# data
		if self.computername:
			if not self.roomname:
				self.roomname = autots.computer_room(self.computername)
			self.ip = autots.computer_get_ip(self.computername)
			self.mac = autots.computer_get_mac(self.computername)
			self.ltsconf = autots.computer_get_ltsconf(self.computername)
		else:
			self.ip = "1.2.3.4"
			self.mac = "00:11:22:33:44:55"
			self.ltsconf = {}

		# window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_default_size(300, 300)
		self.window.set_modal(True)
		self.window.set_transient_for(globals()['room_windows'][self.roomname].window)

		self.window.set_title(self.computername or _("New Computer"))
		self.window.connect("delete-event", self.close)
		
		f = gtk.Frame(label=_("Computer Properties"))
		box = gtk.VBox(False,0)
		f.add(box)

		wbox = gtk.VBox()
		wbox.add(f)
		self.window.add(wbox)
		
		# lts.conf list
		sw = gtk.ScrolledWindow()

		self.liststore = gtk.ListStore(str,str,bool)

		for opt in self.ltsconf:
			self.liststore.append([opt,self.ltsconf[opt],True])

		self.treeview = gtk.TreeView(self.liststore)

		rend1 = gtk.CellRendererText()
		rend1.set_data("column", 0)

		rend2 = gtk.CellRendererText()
		rend2.connect("edited", self.on_cell_edited)
		rend2.set_data("column", 1)

		self.treeview.append_column(gtk.TreeViewColumn(_('Option'), rend1, text=0))
		self.treeview.append_column(gtk.TreeViewColumn(_('Value'), rend2, text=1, editable=2))

		sw.add(self.treeview)

		# window content
		table = gtk.Table(3, 2, False)

		if not self.computername:
			self.ename = gtk.Entry()
			self.ename.set_text(_("new_computer"))
			table.attach(gtk.Label(_('Name')), 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
			table.attach(self.ename, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)	

		self.eip = gtk.Entry()
		self.eip.set_text(self.ip)
		table.attach(gtk.Label(_('IP')), 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(self.eip, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.emac = gtk.Entry()
		self.emac.set_text(self.mac)
		table.attach(gtk.Label(_('MAC')), 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(self.emac, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		box.pack_start(table,False,False,padding=5)

		frame = gtk.Frame(label=_("lts.conf options"))
		fbox = gtk.VBox()
		frame.add(fbox)
		fbox.add(sw)
		
		fbbox = gtk.HBox()

		button1 = gtk.Button(stock=gtk.STOCK_ADD)
		button1.connect("clicked", self.add)
		fbbox.add(button1)

		button2 = gtk.Button(stock=gtk.STOCK_REMOVE)
		button2.connect("clicked", self.remove)
		fbbox.add(button2)

		fbox.pack_start(fbbox,False,False)

		box.pack_start(frame,True,True)

		hbox = gtk.HBox(False,0)

		ok = gtk.Button(stock=gtk.STOCK_OK)
		ok.connect("clicked", self.ok)
		hbox.add(ok)

		cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
		cancel.connect("clicked", self.close)
		hbox.add(cancel)

		wbox.pack_start(gtk.HSeparator(),False,False,padding=5)
		wbox.pack_start(hbox,False,False)

		self.window.show_all()
	
	def on_cell_edited(self,cell, pathstr, new_text):
		iter = self.liststore.get_iter_from_string(pathstr)
		path = self.liststore.get_path(iter)
		option = self.liststore.get_value(iter,0)

		#autots.computer_set_ltsconf(self.computername, option, new_text)
		self.ltsconf[option] = new_text
		self.liststore.set(iter, 1, new_text)
	
	def add(self, button):
		dialog = gtk.Dialog(_("Add Option"), self.window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

		table = gtk.Table(2, 2, False)

		eopt = gtk.Entry()
		table.attach(gtk.Label(_('Option')), 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(eopt, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		evalue = gtk.Entry()
		table.attach(gtk.Label(_('Value')), 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(evalue, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		dialog.vbox.pack_start(table, True, True, 0)
		dialog.vbox.show_all()

		again=True

		while again and dialog.run()==gtk.RESPONSE_OK:
			if not eopt.get_text():
				complain(_("Missing option name"), self.window)
			else:
				iter = self.liststore.append()
				self.liststore.set(iter,0,eopt.get_text(),1,evalue.get_text(),2,True)
				#autots.computer_set_ltsconf(self.computername, eopt.get_text(),evalue.get_text())
				self.ltsconf[eopt.get_text()] = evalue.get_text()
				again=False

		dialog.destroy()

	def remove(self,button):
		model,iter = self.treeview.get_selection().get_selected()

		if iter:
			option = model.get_value(iter,0)
			#autots.computer_set_ltsconf(self.computername, option,None)
			if self.ltsconf.has_key(option):
				del self.ltsconf[option]

			model.remove(iter)

	def ok(self,*data):
		if not self.computername:
			try:
				computer = autots.parse_hostname(self.ename.get_text())
				autots.computer_create(self.roomname, computer, self.eip.get_text(), self.emac.get_text())
				autots.computer_set_ltsconf_all(computer,self.ltsconf)
				globals()['room_windows'][autots.computer_room(computer)].computer_update_button(computer)
				self.close(None)
			except autots.Error,e:
				complain(str(e), self.window)
		else:
			try:
				autots.computer_set_ip(self.computername,self.eip.get_text())
				autots.computer_set_mac(self.computername,self.emac.get_text())
				autots.computer_set_ltsconf_all(self.computername,self.ltsconf)
				self.close(None)
			except autots.Error,e:
				complain(str(e),self.window)
	
	def close(self,widget,*data):
		self.window.destroy()

class room_window:
	def __init__(self, roomname):
		self.roomname = roomname

		# window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)

		self.window.set_title(self.roomname)
		self.window.connect("delete-event", self.close)
		self.window.set_default_size(300, 300)

		box = gtk.VBox(False,0)
		self.window.add(box)

		# window content
		table = gtk.Table(2, 2, False)
		box.pack_start(table, True, True, 0)

		hbox=gtk.HBox();
		box.pack_start(hbox, False, False, 0)

		# buttons
		button1 = gtk.Button(stock=gtk.STOCK_ADD)
		button1.connect("clicked", self.computer_add)
		hbox.add(button1)

		button2 = gtk.Button(stock=gtk.STOCK_CLOSE)
		button2.connect("clicked", self.close)
		hbox.add(button2)

		# room
		self.layout = gtk.Layout(None, None)
		self.layout.set_size(600, 600)
		self.layout.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT |
			gtk.DEST_DEFAULT_DROP, [("computer",0,1)], gtk.gdk.ACTION_MOVE)
		self.layout.connect("drag_data_received", self.dnd_get)
		self.layout.connect('button-press-event',self.room_press)
		table.attach(self.layout, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		vScrollbar = gtk.VScrollbar(None)
		vAdjust = self.layout.get_vadjustment()
		vScrollbar.set_adjustment(vAdjust)
		table.attach(vScrollbar, 1, 2, 0, 1, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)

		hScrollbar = gtk.HScrollbar(None)
		hAdjust = self.layout.get_hadjustment()
		hScrollbar.set_adjustment(hAdjust)
		table.attach(hScrollbar, 0, 1, 1, 2, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)

		# computers
		self.computer_buttons = {}
		
		for computer in autots.room_computers(self.roomname):
			self.computer_update_button(computer)

		self.window.show_all()

	def computer_update_button(self,computer):
		if autots.computer_exists(computer) and autots.computer_room(computer)==self.roomname:
			pos = autots.computer_get_position(computer)

			if self.computer_buttons.has_key(computer):
				# update position
				self.layout.move(self.computer_buttons[computer],pos[0],pos[1])
			else:
				# create computer button
				self.computer_buttons[computer] = gtk.Button()

				label = gtk.Label(computer)
				image = gtk.Image()
				image.set_from_file( os.path.join(autots.root,'computer.png') )

				box = gtk.VBox()
				box.add(image)
				box.add(label)

				self.computer_buttons[computer].add(box)

				self.computer_buttons[computer].drag_source_set(gtk.gdk.BUTTON1_MASK, [("computer",0,1)],
					gtk.gdk.ACTION_MOVE)
				self.computer_buttons[computer].connect("drag_data_get", self.dnd_send, computer)
				self.computer_buttons[computer].connect('button-press-event',self.computer_press, computer)
				self.computer_buttons[computer].connect('clicked',self.computer_edit, computer)
				self.computer_buttons[computer].show_all()

				# place button in room
				self.layout.put(self.computer_buttons[computer], pos[0], pos[1])
		else:
			if self.computer_buttons.has_key(computer):
				# delete button
				self.computer_buttons[computer].destroy()
				del self.computer_buttons[computer]

	# Room actions

	def room_rename(self, nname):
		self.roomname = nname
		self.window.set_title(nname)
	
	def close(self, widget, *data):
		self.window.destroy()
		del globals()['room_windows'][self.roomname]

	# Computer actions
	
	def computer_add(self, button):
		computer_window(None,self.roomname)

	def computer_rename(self,item,computer):
		dialog = gtk.Dialog(_("Rename"), self.window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
		entry = gtk.Entry()
		entry.set_text(computer)
		entry.show()
		dialog.vbox.pack_start(entry, True, True, 0)
		r=dialog.run()

		again=True

		while r==gtk.RESPONSE_OK and again:
			try:
				autots.computer_rename(computer,entry.get_text());
				self.computer_update_button(computer)
				self.computer_update_button(entry.get_text())
				again=False
			except autots.Error,e:
				complain(str(e),self.window)
				r=dialog.run()

		dialog.destroy()

	def computer_remove(self, item, computer):
		if ask(_("Really remove computer %s?") % computer, self.window) == gtk.RESPONSE_YES:
			try:
				autots.computer_remove(computer)
			except autots.Error,e:
				complain(str(e), self.window)

			#self.computer_buttons[computer].destroy()
			#del self.computer_buttons[computer]
			self.computer_update_button(computer)

			if globals()['drag_pos'].has_key(computer):
				del globals()['drag_pos'][computer]

	def computer_edit(self, item, computer):
		computer_window(computer)

	# Press events

	def computer_press(self,obj,event, computer):
		globals()['drag_pos'][computer] = self.computer_buttons[computer].get_pointer()

		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
			menu = gtk.Menu()

			edit = gtk.ImageMenuItem(gtk.STOCK_EDIT)
			edit.show()
			edit.connect("activate", self.computer_edit, computer)
			menu.append(edit)

			rename = gtk.MenuItem(_("Rename"))
			rename.show()
			rename.connect("activate", self.computer_rename, computer)
			menu.append(rename)

			remove = gtk.ImageMenuItem(gtk.STOCK_REMOVE)
			remove.show()
			remove.connect("activate", self.computer_remove, computer)
			menu.append(remove)

			menu.popup(None,None,None,event.button,event.time)

	def room_press(self,obj,event):
		if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
			menu = gtk.Menu()

			add = gtk.ImageMenuItem(gtk.STOCK_ADD)
			add.show()
			add.connect("activate", self.computer_add)
			menu.append(add)

			close = gtk.ImageMenuItem(gtk.STOCK_CLOSE)
			close.show()
			close.connect("activate", self.close)
			menu.append(close)

			menu.popup(None,None,None,event.button,event.time)

	# drag and drop

	def dnd_send(self, widget, context, selection, targetType, time, computer):
		if targetType == 1:
			selection.set(selection.target, 8, computer)

	def dnd_get(self, widget, context, x, y, selection, targetType, time):
		if targetType == 1:
			try:
				computer= selection.data
				oldroom = autots.computer_room(computer)
				autots.computer_move(computer, self.roomname)

				posx = int(round(int(x+self.layout.get_hadjustment().value)
					-globals()['drag_pos'][computer][0],-1))

				posy = int(round(int(y+self.layout.get_vadjustment().value)
					-globals()['drag_pos'][computer][1],-1))

				autots.computer_set_position(computer, (posx,posy))

				self.computer_update_button(computer)
				globals()['room_windows'][oldroom].computer_update_button(computer)
			except autots.Error,e:
				complain(str(e),self.window)

class config_window:
	def __init__(self):
		# window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_default_size(300, 300)
		self.window.set_modal(True)

		self.window.set_title(_("Configuration"))
		self.window.connect("delete-event", self.close)
		
		wbox = gtk.VBox()
		self.window.add(wbox)

		# paths		
		pf = gtk.Frame(label=_("Paths"))
		wbox.add(pf)

		table = gtk.Table(4, 2, False)
		pf.add(table)

		self.ehosts = gtk.Entry()
		self.ehosts.set_text(autots.get_config("hosts"))
		table.attach(gtk.Label(_('hosts file')), 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(self.ehosts, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.edhcpd = gtk.Entry()
		self.edhcpd.set_text(autots.get_config("dhcpd.conf"))
		table.attach(gtk.Label(_('dhcpd.conf')), 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(self.edhcpd, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.elts = gtk.Entry()
		self.elts.set_text(autots.get_config("lts.conf"))
		table.attach(gtk.Label(_('lts.conf')), 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(self.elts, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.einit = gtk.Entry()
		self.einit.set_text(autots.get_config("dhcpd_init_script"))
		table.attach(gtk.Label(_('dhcpd init script')), 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		table.attach(self.einit, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		# server.py
		sf = gtk.Frame(label=_("server.py options"))
		wbox.add(sf)

		stable = gtk.Table(4, 2, False)
		sf.add(stable)

		self.eprefix = gtk.Entry()
		self.eprefix.set_text(autots.get_config("guest_prefix"))
		stable.attach(gtk.Label(_('guest prefix')), 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.eprefix, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.egroup = gtk.Entry()
		self.egroup.set_text(autots.get_config("guest_group"))
		stable.attach(gtk.Label(_('guest group')), 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.egroup, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.egroups = gtk.Entry()
		self.egroups.set_text(autots.get_config("guest_groups"))
		stable.attach(gtk.Label(_('additional groups(comma separated)')), 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.egroups, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.ehomes = gtk.Entry()
		self.ehomes.set_text(autots.get_config("guest_homes"))
		stable.attach(gtk.Label(_('directory containing guest homes')), 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.ehomes, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.emodel = gtk.Entry()
		self.emodel.set_text(autots.get_config("guest_model_home"))
		stable.attach(gtk.Label(_('model home directory')), 0, 1, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.emodel, 1, 2, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.eiface = gtk.Entry()
		self.eiface.set_text(autots.get_config("server_interface"))
		stable.attach(gtk.Label(_('server interface')), 0, 1, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.eiface, 1, 2, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.eport = gtk.Entry()
		self.eport.set_text(autots.get_config("server_port"))
		stable.attach(gtk.Label(_('server port')), 0, 1, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.eport, 1, 2, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.exsession = gtk.Entry()
		self.exsession.set_text(autots.get_config("ltsp4_xsession"))
		stable.attach(gtk.Label(_('xsession (only LTSP4)')), 0, 1, 8, 9, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		stable.attach(self.exsession, 1, 2, 8, 9, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		hbox = gtk.HBox(False,0)

		ok = gtk.Button(stock=gtk.STOCK_OK)
		ok.connect("clicked", self.ok)
		hbox.add(ok)

		cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
		cancel.connect("clicked", self.close)
		hbox.add(cancel)

		wbox.pack_start(gtk.HSeparator(),False,False,padding=5)
		wbox.pack_start(hbox,False,False)

		self.window.show_all()

	def ok(self,*data):
		try:
			autots.set_config("hosts",self.ehosts.get_text())
			autots.set_config("dhcpd.conf",self.edhcpd.get_text())
			autots.set_config("lts.conf",self.elts.get_text())
			autots.set_config("dhcpd_init_script",self.einit.get_text())
			autots.set_config("guest_prefix",self.eprefix.get_text())
			autots.set_config("guest_group",self.egroup.get_text())
			autots.set_config("guest_groups",self.egroups.get_text())
			autots.set_config("guest_homes",self.ehomes.get_text())
			autots.set_config("guest_model_home",self.emodel.get_text())
			autots.set_config("server_interface",self.eiface.get_text())
			autots.set_config("server_port",self.eport.get_text())
			autots.set_config("ltsp4_xsession",self.exsession.get_text())

			self.close(None)
		except autots.Error,e:
			complain(str(e),self.window)
	
	def close(self,widget,*data):
		self.window.destroy()

class network_window:
	"""The network window that lists all rooms in a TreeView"""
	def __init__(self):
		# create ListStore
		self.liststore = gtk.ListStore(str)

		for room in autots.rooms():
			self.liststore.append([room])

		# create TreeView
		self.treeview = gtk.TreeView(self.liststore)
		#self.treeview.set_reorderable(True)
		self.treeview.append_column(gtk.TreeViewColumn(_('room'), gtk.CellRendererText(), text=0))

		self.treeview.connect('row-activated',self.room_doubleclicked)

		# create Buttons
		hbox = gtk.HBox()

		edit = gtk.Button(stock=gtk.STOCK_EDIT)
		edit.connect('clicked', self.edit)
		hbox.add(edit)

		rename = gtk.Button(_('Rename'))
		rename.connect('clicked', self.rename)
		hbox.add(rename)

		add = gtk.Button(stock=gtk.STOCK_ADD)
		add.connect('clicked', self.add)
		hbox.add(add)

		remove = gtk.Button(stock=gtk.STOCK_REMOVE)
		remove.connect('clicked', self.remove)
		hbox.add(remove)

		config = gtk.Button(stock=gtk.STOCK_PREFERENCES)
		config.connect('clicked', self.config)
		hbox.add(config)

		close = gtk.Button(stock=gtk.STOCK_CLOSE)
		close.connect('clicked', self.close)
		hbox.add(close)

		# create window content
		box = gtk.VBox(False,0)

		box.pack_start(gtk.Label(_('Room List')), False)

		sw = gtk.ScrolledWindow()
		box.pack_start(sw, True)
		sw.add(self.treeview)

		box.pack_start(hbox, False)

		# create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_title(_("Network"))
		self.window.connect("destroy", self.close)
		self.window.set_default_size(400, 200)

		self.window.add(box)
		self.window.show_all()

	def rooms_reordered(self, **a):
		print a

	def room_doubleclicked(self, treeview, path, view_column):
		room = treeview.get_model().get_value(treeview.get_model().get_iter(path), 0)

		if not globals()['room_windows'].has_key(room):
			globals()['room_windows'][room] = room_window(room)

	def edit(self, button):
		try:
			treeselection = self.treeview.get_selection()
			(model, iter) = treeselection.get_selected()
			room = model.get_value(iter, 0)
		except:
			pass
		else:
			if not globals()['room_windows'].has_key(room):
				globals()['room_windows'][room] = room_window(room)

	def rename(self, button):
		try:
			treeselection = self.treeview.get_selection()
			(model, iter) = treeselection.get_selected()
			room = model.get_value(iter, 0)
		except:
			pass
		else:
			dialog = gtk.Dialog(_("Rename"), self.window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
			entry = gtk.Entry()
			entry.set_text(room)
			entry.show()
			dialog.vbox.pack_start(entry, True, True, 0)
			
			run=True

			while run and dialog.run()==gtk.RESPONSE_OK:
				try:
					newroom = autots.parse_roomname(entry.get_text())
					autots.room_rename(room, newroom)
					run=False

					if globals()['room_windows'].has_key(room):
						globals()['room_windows'][newroom] = globals()['room_windows'][room]
						del globals()['room_windows'][room]
						globals()['room_windows'][newroom].room_rename(newroom)

					model.set_value(iter, 0, newroom)
				except autots.Error,e:
					complain(str(e), self.window)

			dialog.destroy()

	def add(self, button):
		dialog = gtk.Dialog(_("Add"), self.window, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, (gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
		entry = gtk.Entry()
		entry.set_text(_('new_room'))
		dialog.vbox.pack_start(entry, True, True, 0)
		dialog.show_all()

		run=True

		while run and dialog.run()==gtk.RESPONSE_OK:
			try:
				newroom = autots.parse_roomname(entry.get_text())
				autots.room_create(newroom)
				run=False

				self.liststore.append([newroom])
			except autots.Error,e:
				complain(str(e), self.window)
		
		dialog.destroy()

	def remove(self, button):
		try:
			treeselection = self.treeview.get_selection()
			(model, iter) = treeselection.get_selected()
			room = model.get_value(iter, 0)
		except:
			pass
		else:
			if ask(_("Really remove room %s?") % room, self.window) == gtk.RESPONSE_YES:
				if globals()['room_windows'].has_key(room):
					globals()['room_windows'][room].close(None)

				autots.room_remove(room)

				model.remove(iter)

	def config(self,button):
		config_window()

	def close(self, widget, *data):
		r=ask(_("Save?"), self.window)

		if r==gtk.RESPONSE_YES:
			try:
				autots.save()
				autots.update_hosts()
				autots.update_dhcpd()
				autots.update_ltsconf()

				if ask(_("Do you want the dhcp server to be restarted now? (necessary to apply changes)"), self.window)==gtk.RESPONSE_YES:
					autots.dhcpd_restart()
			except autots.Error,e:
				complain(str(e), self.window)
			else:
				gtk.main_quit()

		elif r==gtk.RESPONSE_NO:
			gtk.main_quit()

if __name__ == "__main__":
	try:
		autots.load()
	except autots.Error,e:
		dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, str(e)+"\n"+_("A new empty list is being created."))
		dialog.run()
		dialog.destroy()
		autots.purge()

	network_window()
	gtk.main()
