#!/usr/bin/env python
#
# Server program for autots
#
# Copyright (C) 2008-2009 Maximilian Hoegner
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

#
# Server for LTSP-4 clients that provides automatic login and does session initializing, 
# monitoring and finalizing. Need src/connect.pl on the client.
#

import logging
logging.getLogger('').addHandler(logging.StreamHandler())
logging.getLogger('').setLevel(logging.INFO)

import autots
_ = autots._

import sys

try:
	autots.load()
except autots.Error,e:
	logging.getLogger('').error(str(e))
	sys.exit(1)

group = autots.get_config("guest_group")
groups = [i.strip() for i in autots.get_config("guest_groups").split(",")]
homes = autots.get_config("guest_homes")
model_home = autots.get_config("guest_model_home")
bind = (autots.get_config("server_interface"), int(autots.get_config("server_port")))
username_prefix = autots.get_config("guest_prefix")
xsession = autots.get_config("ltsp4_xsession")

import os,subprocess

def hostname(address):
	ip = SocketServer.socket.gethostbyname(address)
	list = autots.computers_with_property('IP',ip)
	if len(list)==0:
		raise Exception, _("no computer with IP %s found") % address
	else:
		return list[0][1]

def create_account(username):
	if subprocess.call(["id",username])==0:
		return True

	add = ["useradd"]
	add.extend(["-d", os.path.join(homes,username) ])
	add.extend(["-g", group ])

	if len(groups)>0:
		add.extend(["-G", ",".join(groups) ])

	add.append(username)

	if not subprocess.call(add)==0:
		raise Exception, _("adding user %s failed") % username

	return subprocess.call(["id",username])==0

def create_home(username):
	subprocess.call(["mkdir","-p","--", os.path.join(homes,username) ])

	if not subprocess.call(["rm","-rf","--", os.path.join(homes,username) ])==0:
		raise Exception, _("Error deleting home directory of %s") % username

	if not subprocess.call(["cp","-a", "--", model_home, os.path.join(homes,username) ])==0:
		raise Exception, _("Error copying home directory of %s") % username
		
	if not subprocess.call(["chmod", "--", "0700", os.path.join(homes,username) ])==0:
		raise Exception, _("Error setting rights on home directory of %s") % username
		
	if not subprocess.call(["chown","-R", "--", username+':'+group, os.path.join(homes,username) ])==0:
		raise Exception, _("Error setting owner and group of home directory of %s") % username

def magic_cookie(username, cookie):
	# add magic cookie to xauth
	if not subprocess.call(["su","-c","XAUTHORITY=~/.Xauthority xauth add "+cookie, "--", username])==0:
#	if not subprocess.call(["sudo","-H","-u",username,"sh","-c","XAUTHORITY=~/.Xauthority xauth add "+cookie])==0:
		raise Exception, _("xauth failed for %s") % username

def terminate_session(username):
	subprocess.call(["pkill", "-u", username])
	time.sleep(2)
	subprocess.call(["pkill", "-9", "-u", username])

	cnt=0

	while not subprocess.call(["pgrep", "-u", username])==1:
		time.sleep(1)

		cnt +=1

		if cnt>10:
			raise Exception, _("pkill -9 didn't succeed for %s") % username

	# unmount filesystems
	for i in open('/proc/mounts'):
		d = i.split(" ")[1].decode("string-escape") # get directory

		if d.find(os.path.join(homes,username,''))==0:
			subprocess.call(["umount","-fl","--", d])

	# delete home directory
	if not subprocess.call(["rm","-rf","--", os.path.join(homes,username) ])==0:
		raise Exception, _("Error deleting home directory of %s") % username

import SocketServer, time, threading, signal

guest_list = {}
guest_list_lock = threading.Lock()

class RequestHandler(SocketServer.BaseRequestHandler):
	def setup(self):
		self.logger = logging.getLogger(self.client_address[0])

		self.logger.info(_('%s connected'), self.client_address[0])

		self.request.settimeout(10)
		self.f=self.request.makefile()

	def handle(self):
		try:
			h = hostname(self.client_address[0])
		except Exception,e:
			self.logger.warning(_("Computer %s not accepted")+': ',str(e),self.client_address[0])
			return

		username = username_prefix + h

		guest_list_lock.acquire()

		if username in guest_list:
			self.logger.warning(_("Computer %s is already having a session"), self.client_address[0])
			guest_list_lock.release()
			return

		guest_list[username] = True

		guest_list_lock.release()

		try:
			create_account(username)
		except Exception,e:
			self.logger.error(_("Account for %s couldn't be created")+': '+str(e),username)
			guest_list_lock.acquire()
			del guest_list[username]
			guest_list_lock.release()
			return

		try:
			create_home(username)
		except Exception,e:
			self.logger.error(_("Home directory for %s couldn't be created")+': '+str(e),username)
			guest_list_lock.acquire()
			del guest_list[username]
			guest_list_lock.release()
			return

		# get magic cookie
		try:
			self.request.sendall(h+"\n")
			display = self.f.readline().strip()
			cookie = self.f.readline().strip()
		except Exception,e:
			self.logger.error(_("Couldn't fetch magic cookie for %s")+': '+str(e),username)
			guest_list_lock.acquire()
			del guest_list[username]
			guest_list_lock.release()
			return

		try:
			magic_cookie(username,cookie)
		except Exception,e:
			self.logger.error(_("Error giving magic cookie to %s")+': '+str(e),username)
			guest_list_lock.acquire()
			del guest_list[username]
			guest_list_lock.release()
			return

		self.running=True

		subprocess.call(["pkill", "-9", "-u", username])

#		self.p=subprocess.Popen([
#			"sudo",
#			"-H",
#			"-u",
#			username,
#			"sh",
#			"-c",
#			'cd ~; XAUTHORITY=~/.Xauthority DISPLAY="%s" %s' % (display,xsession)
#		])
		self.p=subprocess.Popen([
			"su",
			"-c",
			'cd ~; XAUTHORITY=~/.Xauthority DISPLAY="%s" %s' % (display,xsession),
			"--",
			username
		])

		mon=threading.Thread(target=self.monitor)
		mon.start()

		self.p.wait()
		self.running=False

		try:
			terminate_session(username)
		except Exception,e:
			self.logger.error(_("terminating session for %s failed")+': '+str(e),username)

		guest_list_lock.acquire()
		del guest_list[username]
		guest_list_lock.release()

		mon.join()

	def monitor(self):
		logger = logging.getLogger(self.client_address[0])

		try:
			while self.running:
				# test connection to client
				self.request.sendall("\n")
				self.f.readline()

				time.sleep(10)

			logger.debug(_('session of %s ended'), self.client_address[0])
		except Exception,e:
			if self.running:
				logger.warning(_('connection to %s broken, killing session')+': '+str(e), self.client_address[0])

				subprocess.call(["kill", str(self.p.pid)])
				time.sleep(2)

			if self.running:
				subprocess.call(["kill", "-9", str(self.p.pid)])
				time.sleep(2)

			if self.running:
				logger.error(_("killing session doesn't succeed for %s!"), self.client_address[0])

	def finish(self):
		self.logger.info(_('%s disconnected'), self.client_address[0])

class ThreadingTCPServerReuse(SocketServer.ThreadingTCPServer):
	def __init__(self,server_address,RequestHandlerClass):
		self.allow_reuse_address=True
		SocketServer.ThreadingTCPServer.__init__(self,server_address,RequestHandlerClass)

def terminate(sig,*args):
	logging.getLogger('').info(_("received signal %d") % sig)
	raise autots.Exception

signal.signal(signal.SIGTERM, terminate)
signal.signal(signal.SIGINT, terminate)

def run(server):
	try:
		server.serve_forever()
	except:
		logging.getLogger('').info(_("kill client sessions (SIGTERM)"))

		guest_list_lock.acquire()
		gl = [i for i in guest_list]
		guest_list_lock.release()

		for username in gl:
			subprocess.call(["pkill", "-u", username])

		time.sleep(3)

		logging.getLogger('').info(_("kill client sessions (SIGKILL)"))
		for username in gl:
			subprocess.call(["pkill", "-9", "-u", username])
	

if __name__=="__main__":
	server = ThreadingTCPServerReuse(bind, RequestHandler)
	run(server)
