#!/usr/bin/env python
#
# Server program for autots
#
# Copyright (C) 2009 Maximilian Hoegner
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

#
# Server for LTSP-5 clients that provides automatic login and initializes
# sessions. So far no monitoring. Need python-openssl installed on server 
# and src/099-remoteltscfg as ltsp plugin in /usr/share/ltsp/plugins/
# ltsp-build-client/Ubuntu and needs the ltsp client root to be built with
# ltsp-build-client --rltscfg.
#

from server4 import *
import OpenSSL, random, socket

_ = autots._

pwchars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
pwchars_limit = len(pwchars)-1

def create_cert():
	pkey = OpenSSL.crypto.PKey()
	pkey.generate_key(OpenSSL.crypto.TYPE_RSA, 1024)
	cert = OpenSSL.crypto.X509()
	cert.set_pubkey(pkey)
	cert.gmtime_adj_notBefore(0)
	cert.gmtime_adj_notAfter(60*60*24*365*50)
	return pkey,cert

def create_password():
	r = ''
	for i in xrange(30):
		r += pwchars[random.randint(0,pwchars_limit)]

	return r

class RequestHandler5(RequestHandler):
	def setup(self):
		self.logger = logging.getLogger(self.client_address[0])

		self.logger.info(_('%s connected'), self.client_address[0])

		self.request.settimeout(30)

	def handle(self):
		try:
			h = hostname(self.client_address[0])
		except Exception,e:
			self.logger.warning(_("Computer %s not accepted")+': '+str(e),self.client_address[0])
			return

		username = username_prefix + h

		guest_list_lock.acquire()

		if username in guest_list:
			self.logger.warning(_("conflict: Computer %s is currently being initialized"), self.client_address[0])
			guest_list_lock.release()
			return

		guest_list[username] = True

		guest_list_lock.release()

		try:
			create_account(username)
		except Exception,e:
			self.logger.error(_("Account for %s couldn't be created")+': '+str(e),username)
			guest_list_lock.acquire()
			del guest_list[username]
			guest_list_lock.release()
			return

		try:
			terminate_session(username)
		except Exception,e:
			self.logger.warning(_("Session for %s couldn't be terminated")+': '+str(e),username)

		try:
			create_home(username)
		except Exception,e:
			self.logger.error(_("Home directory for %s couldn't be created")+': '+str(e),username)
			guest_list_lock.acquire()
			del guest_list[username]
			guest_list_lock.release()
			return

		# set new password for user
		try:
			password = create_password()

			p=subprocess.Popen("chpasswd",stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
			out,err = p.communicate(username+":"+password+"\n")

			if not out=='':
				self.logger.info(_("chpasswd(stdout)")+': '+out)
			if not err=='':
				self.logger.info(_("chpasswd(stderr)")+': '+err)

			p.wait()

		except Exception,e:
			self.logger.error(_("setting password for %s failed")+': '+str(e),username)

		# give password to client
		try:
			self.request.sendall("export LDM_USERNAME=\""+username+"\"\n"+"export LDM_PASSWORD=\""+password+"\"\n")
		except Exception,e:
			self.logger.error(_("giving password to %s failed")+': '+str(e),h)

		del password

		guest_list_lock.acquire()
		del guest_list[username]
		guest_list_lock.release()

class ThreadingTCPServerReuseSSL(SocketServer.ThreadingTCPServer):
	def __init__(self,server_address,RequestHandlerClass):
		self.allow_reuse_address=True
		SocketServer.ThreadingTCPServer.__init__(self,server_address,RequestHandlerClass)

		pkey,cert=create_cert()
		ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
		ctx.use_certificate(cert)
		ctx.use_privatekey(pkey)

		sock = socket.socket(self.address_family, self.socket_type)
	        self.socket = OpenSSL.SSL.Connection(ctx , sock)

		self.server_bind()
		self.server_activate()

		host, port = self.socket.getsockname()[:2]
		self.server_name = socket.getfqdn(host)
		self.server_port = port

if __name__=="__main__":
	server = ThreadingTCPServerReuseSSL(bind, RequestHandler5)
	run(server)
